import subprocess


def test_compile_vite_apps():
    p = subprocess.run(["vt", "compile", "-y"], stdout=subprocess.PIPE)
    terminal_output = p.stdout.decode("utf-8")
    assert "Compilation complete." in terminal_output


def test_list_vite_apps():
    p = subprocess.run(["vt", "list"], stdout=subprocess.PIPE)
    terminal_output = p.stdout.decode("utf-8")
    assert (
        "\x1b[92mfrontend/dist/assets\x1b[0m \x1b[1m=>\x1b[0m \x1b[92mapp_flask/vt/frontend/\x1b[0m"
        in terminal_output
    )


def test_app_flask(client):
    response = client.get("/")
    assert response.status_code == 200



HTTP_HEADERS = (
    "Accept",
    "Accept-Language",
    "Authorization",
    "Content-Language",
    "Content-Type",
    "DPR",
    "Downlink",
    "Origin",
    "Save-Data",
    "Viewport-Width",
    "Width",
    "X-AUTH-TOKEN",
    "X-CSRF-TOKEN",
    "X-REQUESTED-WITH",
    "X-Auth-Token",
    "X-Csrf-Token",
    "X-Requested-With",
    "x-Auth-Token",
    "x-Csrf-Token",
    "x-Requested-With",
    "x-auth-token",
    "x-csrf-token",
    "x-requested-with",
)



import typing as t
from pathlib import Path
from tomllib import loads  # type: ignore


class PyProjectConfig:
    cwd: Path
    pyproject: Path
    vt_config: t.Dict[str, t.Any]

    npm_exec: str
    npx_exec: str
    serve_app: str
    vite_apps: t.List[t.Dict[str, t.Any]]

    def __init__(self) -> None:
        self.cwd = Path.cwd()
        self.pyproject = self.cwd / "pyproject.toml"
        self.load_pyproject()

    def load_pyproject(self) -> None:
        if not self.pyproject.exists():
            raise FileNotFoundError("pyproject.toml not found.")

        pyproject_raw = loads(str(self.pyproject.read_text()))
        self.vt_config = pyproject_raw.get("tool", {}).get("vite_transporter", {})
        self.npm_exec = self.vt_config.get("npm_exec", "npm")
        self.npx_exec = self.vt_config.get("npx_exec", "npx")
        self.serve_app = self.vt_config.get("serve_app", "app")
        self.vite_apps = self.vt_config.get("vite_apps", [])

    def __enter__(self) -> "PyProjectConfig":
        return self

    def __exit__(self, exc_type: t.Any, exc_val: t.Any, exc_tb: t.Any) -> None:
        return None



import sys
from pathlib import Path

import pytest

sys.path.append(str(Path(__file__).parent.parent))


@pytest.fixture(scope="session")
def app():
    from app_flask import create_app

    app = create_app()
    yield app


@pytest.fixture(scope="session")
def client(app):
    return app.test_client()



import sys
import typing as t

from .sprinkles import Sprinkles


def list_vite_apps(vite_apps_found: t.List[t.Dict[str, t.Any]]) -> None:
    print("")
    if not vite_apps_found:
        print(
            f" {Sprinkles.WARNING}No vite apps found in pyproject.toml{Sprinkles.END}"
        )
    else:
        for app in vite_apps_found:
            print(
                f"{Sprinkles.OKGREEN}{app.get('vite_app')}/dist/assets{Sprinkles.END} "
                f"{Sprinkles.BOLD}=>{Sprinkles.END} "
                f"{Sprinkles.OKGREEN}{app.get('serve_app')}/vt/{app.get('vite_app')}/{Sprinkles.END}"
            )
    print("")
    sys.exit(0)



from .argument_parser import ArgumentParser
from .compile_vite_apps import compile_vite_apps
from .list_vite_apps import list_vite_apps
from .load_vite_app import load_vite_apps
from .pyproject_config import PyProjectConfig
from .sprinkles import Sprinkles
from .subparsers import subparsers

__all__ = [
    "ArgumentParser",
    "compile_vite_apps",
    "list_vite_apps",
    "load_vite_apps",
    "PyProjectConfig",
    "Sprinkles",
    "subparsers",
]



import sys
import typing as t
from pathlib import Path

from vite_transporter.elements import BodyContent, ScriptTag, LinkTag
from vite_transporter.globals import HTTP_HEADERS
from vite_transporter.utilities import Sprinkles

if "quart" in sys.modules:
    from markupsafe import Markup
    from quart import Quart, url_for, send_from_directory, Response
else:
    raise ImportError("Quart is not installed.")


class ViteTransporter:
    app: t.Optional[Quart]
    vt_root_path: Path

    cors_allowed_hosts: t.Optional[t.List[str]]

    def __init__(
        self,
        app: t.Optional[Quart] = None,
        cors_allowed_hosts: t.Optional[t.List[str]] = None,
    ) -> None:
        if app is not None:
            self.init_app(app, cors_allowed_hosts)

    def init_app(
        self, app: Quart, cors_allowed_hosts: t.Optional[t.List[str]] = None
    ) -> None:
        if app is None:
            raise ImportError("No app was passed in.")
        if not isinstance(app, Quart):
            raise TypeError("The app that was passed in is not an instance of Quart")

        self.app = app
        self.cors_allowed_hosts = cors_allowed_hosts

        if "vite_transporter" in self.app.extensions:
            raise ImportError(
                "The app has already been initialized with vite-to-flask."
            )

        self.app.extensions["vite_transporter"] = self
        self.app.config["VTF_APPS"] = {}
        self.vt_root_path = Path(app.root_path) / "vite"

        if not self.vt_root_path.exists():
            print(
                f"{Sprinkles.WARNING}{Sprinkles.BOLD}vt folder not found, a new one was created.{Sprinkles.END}{Sprinkles.END}\n\r"
                f"{Sprinkles.OKCYAN}{self.vt_root_path}{Sprinkles.END}\n\r"
            )
            self.vt_root_path.mkdir()

        for folder in self.vt_root_path.iterdir():
            if folder.is_dir():
                self.app.config["VTF_APPS"].update({folder.name: folder})

        self._load_routes(app)
        self._load_context_processor(app)
        self._load_cors_headers(app, cors_allowed_hosts=self.cors_allowed_hosts)

    def _load_routes(self, app: Quart) -> None:
        @app.route("/--vite--/<vite_app>/<filename>")
        async def __vite__(vite_app: str, filename: str) -> Response:
            return await send_from_directory(self.vt_root_path / vite_app, filename)

    @staticmethod
    def _load_context_processor(app: Quart) -> None:
        @app.context_processor
        async def vt_head_processor() -> t.Dict[str, t.Callable[..., t.Any]]:
            def vt_head(vite_app: str) -> t.Any:
                vite_assets = Path(app.root_path) / "vite" / vite_app
                find_vite_js = vite_assets.glob("*.js")
                find_vite_css = vite_assets.glob("*.css")

                tags: t.List[t.Union[ScriptTag, LinkTag]] = []

                for file in find_vite_js:
                    tags.append(
                        ScriptTag(
                            src=url_for(
                                "__vite__",
                                vite_app=f"{vite_app}",
                                filename=f"{file.name}",
                            ),
                            type_="module",
                        )
                    )

                for file in find_vite_css:
                    tags.append(
                        LinkTag(
                            rel="stylesheet",
                            href=url_for(
                                "__vite__",
                                vite_app=f"{vite_app}",
                                filename=f"{file.name}",
                            ),
                        )
                    )

                return Markup("".join([tag.raw() for tag in tags]))

            return dict(vt_head=vt_head)

        @app.context_processor
        async def vt_body_processor() -> t.Dict[str, t.Callable[..., t.Any]]:
            def vt_body(
                root_id: str = "root",
                noscript_message: str = "You need to enable JavaScript to run this app.",
            ) -> t.Any:
                return BodyContent(root_id, noscript_message)()

            return dict(vt_body=vt_body)

    @staticmethod
    def _load_cors_headers(
        app: Quart, cors_allowed_hosts: t.Optional[t.List[str]] = None
    ) -> None:
        if cors_allowed_hosts:
            print(
                f"\n\r{Sprinkles.WARNING}{Sprinkles.BOLD}vite-transporter is disabling CORS restrictions for:"
                f"{Sprinkles.END}{Sprinkles.END}\n\r"
                f"{Sprinkles.OKCYAN}{', '.join(cors_allowed_hosts)}{Sprinkles.END}\n\r"
            )

            @app.after_request
            async def after_request(response: Response) -> Response:
                response.headers["Access-Control-Allow-Origin"] = ", ".join(
                    cors_allowed_hosts
                )
                response.headers["Access-Control-Allow-Headers"] = ", ".join(
                    HTTP_HEADERS
                )
                response.headers["Access-Control-Allow-Methods"] = "*"
                response.headers["Access-Control-Allow-Credentials"] = "true"
                return response



import argparse
import typing as t

from .sprinkles import Sprinkles


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
        super(ArgumentParser, self).__init__(*args, **kwargs)
        self.vite_apps: t.List[t.Dict[str, t.Any]] = []

    def print_help(self, file: t.Optional[t.IO[str]] = None) -> None:
        print(
            "\n\r"
            "Usage: vt <option>"
            "\n\r\n\r"
            f" {Sprinkles.OKCYAN}list, ls{Sprinkles.END} => List all vite apps in pyproject.toml"
            "\n\r"
            f" {Sprinkles.OKCYAN}compile (-y){Sprinkles.END} => Attempt to compile all vite apps"
            "\n\r"
            f"  | {Sprinkles.OKCYAN}-y{Sprinkles.END} => Accept all prompts while compiling"
            "\n\r"
            f" {Sprinkles.OKCYAN}-h, --help{Sprinkles.END} => Show the help message and exit"
            "\n\r"
            f" {Sprinkles.OKCYAN}-v, --version{Sprinkles.END} => Show the version and exit"
        )
        print("")



import shutil
import subprocess
import sys
import typing as t
from argparse import Namespace

from .pyproject_config import PyProjectConfig
from .sprinkles import Sprinkles


def compile_vite_apps(
        pyproject_config: PyProjectConfig,
        vite_apps_found: t.List[t.Dict[str, t.Any]],
        parsed_args: Namespace,
) -> None:
    compiler(
        pyproject_config,
        vite_apps_found,
        replace=True if hasattr(parsed_args, "y") and parsed_args.y else False,
    )


def compiler(
        pyproject_config: PyProjectConfig,
        vite_apps_found: t.List[t.Dict[str, t.Any]],
        replace: bool = False,
) -> None:
    print("Compiling Vite apps...")
    vt_dir = pyproject_config.cwd / pyproject_config.serve_app / "vite"

    if vt_dir.exists():
        if not replace:
            prompt = input(
                f"Continuing will replace the contents of \n\r"
                f"{vt_dir} \n\r"
                f"Do you want to continue? (Y/n): "
            )
        else:
            prompt = "y"

        if prompt.lower() == "y" or prompt == "":
            for item in vt_dir.iterdir():
                if item.is_dir():
                    shutil.rmtree(item)
                else:
                    item.unlink()

        else:
            print("Operation aborted.")
            sys.exit(0)

    else:
        vt_dir.mkdir()

    for app in vite_apps_found:
        va_path = pyproject_config.cwd / app.get("vite_app", "")
        va_node_modules = va_path / "node_modules"
        va_dist = va_path / "dist"
        va_assets = va_dist / "assets"

        va_vt_path = vt_dir / app.get("vite_app", "")

        if va_dist.exists() and va_dist.is_dir():
            shutil.rmtree(va_dist)

        if not va_vt_path.exists():
            va_vt_path.mkdir()

        if not va_node_modules.exists():
            subprocess.run([pyproject_config.npm_exec, "install"], cwd=va_path)

        subprocess.run([pyproject_config.npx_exec, "vite", "build", "--mode", "production"], cwd=va_path)

        for item in va_assets.iterdir():
            print(
                f"{Sprinkles.OKGREEN}Copying {item.name} to {va_vt_path}{Sprinkles.END}"
            )

            if item.suffix == ".js":
                with open(va_vt_path / item.name, "w") as f:
                    content = item.read_text()
                    f.write(
                        content.replace("assets/", f"--vite--/{app.get('vite_app')}/")
                    )
            else:
                shutil.copy(item, va_vt_path / item.name)

    print("Compilation complete.")



import typing as t

from markupsafe import Markup


class ScriptTag:
    src: str
    type: t.Optional[str] = None
    async_: bool = False
    defer: bool = False
    crossorigin: t.Optional[str] = None
    integrity: t.Optional[str] = None
    nomodule: bool = False
    referrerpolicy: t.Optional[str] = None

    _src: str
    _type: t.Optional[str] = None
    _async: t.Optional[str] = None
    _defer: t.Optional[str] = None
    _crossorigin: t.Optional[str] = None
    _integrity: t.Optional[str] = None
    _nomodule: t.Optional[str] = None
    _referrerpolicy: t.Optional[str] = None

    def __init__(
        self,
        src: str,
        type_: t.Optional[str] = None,
        async_: bool = False,
        defer: bool = False,
        crossorigin: t.Optional[str] = None,
        integrity: t.Optional[str] = None,
        nomodule: bool = False,
        referrerpolicy: t.Optional[str] = None,
    ) -> None:
        self.src = src
        self.type = type_
        self.async_ = async_
        self.defer = defer
        self.crossorigin = crossorigin
        self.integrity = integrity
        self.nomodule = nomodule
        self.referrerpolicy = referrerpolicy

        self._src = f'src="{self.src}" '
        self._type = f'type="{self.type}" ' if self.type is not None else ""
        self._async = f'async="{str(self.async_).lower()}" ' if self.async_ else ""
        self._defer = "defer " if self.defer else ""
        self._crossorigin = (
            f'crossorigin="{self.crossorigin}" ' if self.crossorigin is not None else ""
        )
        self._integrity = (
            f'integrity="{self.integrity}" ' if self.integrity is not None else ""
        )
        self._nomodule = "nomodule " if self.nomodule else ""
        self._referrerpolicy = (
            f'referrerpolicy="{self.referrerpolicy}" '
            if self.referrerpolicy is not None
            else ""
        )

    def __repr__(self) -> Markup:
        return Markup(
            (
                f"<ScriptTag {self._src}{self._type}"
                f"{self._async}{self._defer}{self._crossorigin}"
                f"{self._integrity}{self._nomodule}{self._referrerpolicy}>"
            ).replace(" >", ">")
        )

    def __str__(self) -> Markup:
        return Markup(self._compile())

    def __call__(self, *args: t.Any, **kwargs: t.Any) -> Markup:
        return Markup(self._compile())

    def raw(self) -> str:
        return self._compile()

    def _compile(self) -> str:
        return (
            f"<script {self._src}{self._type}"
            f"{self._async}{self._defer}{self._crossorigin}"
            f"{self._integrity}{self._nomodule}{self._referrerpolicy}></script>"
        ).replace(" >", ">")



import typing as t

from .pyproject_config import PyProjectConfig


def load_vite_apps(pyproject_config: PyProjectConfig) -> t.List[t.Dict[str, t.Any]]:
    vite_apps = []
    for vite_app in pyproject_config.vite_apps:
        vite_apps.append(
            {
                "vite_app": vite_app,
                "serve_app": pyproject_config.vt_config.get("serve_app"),
            }
        )
    return vite_apps



from .argument_parser import ArgumentParser


def subparsers(parser: ArgumentParser) -> None:
    subparser = parser.add_subparsers()

    compile_ = subparser.add_parser("compile")
    compile_.set_defaults(compile=False)
    compile_.add_argument("-y", action="store_true")

    list_ = subparser.add_parser("list")
    list_.set_defaults(list=False)

    ls = subparser.add_parser("ls")
    ls.set_defaults(ls=False)



import typing as t

from markupsafe import Markup


class LinkTag:
    rel: str
    href: t.Optional[str] = None
    sizes: t.Optional[str] = None
    type: t.Optional[str] = None
    hreflang: t.Optional[str] = None

    _rel: str
    _href: t.Optional[str] = None
    _sizes: t.Optional[str] = None
    _type: t.Optional[str] = None
    _hreflang: t.Optional[str] = None

    def __init__(
        self,
        rel: str,
        href: t.Optional[str] = None,
        sizes: t.Optional[str] = None,
        type_: t.Optional[str] = None,
        hreflang: t.Optional[str] = None,
    ) -> None:
        self.rel = rel
        self.href = href
        self.sizes = sizes
        self.type = type_
        self.hreflang = hreflang

        self._rel = f'rel="{self.rel}" '
        self._href = f'href="{self.href}" ' if self.href is not None else ""
        self._sizes = f'sizes="{self.sizes}" ' if self.sizes is not None else ""
        self._type = f'type="{self.type}" ' if self.type is not None else ""
        self._hreflang = (
            f'hreflang="{self.hreflang}" ' if self.hreflang is not None else ""
        )

    def __repr__(self) -> Markup:
        return Markup(
            f"<LinkTag {self._rel}{self._href}{self._sizes}{self._type}{self._hreflang}>".replace(
                " >", ">"
            )
        )

    def __str__(self) -> Markup:
        return Markup(self._compile())

    def __call__(self, *args: t.Any, **kwargs: t.Any) -> Markup:
        return Markup(self._compile())

    def raw(self) -> str:
        return self._compile()

    def _compile(self) -> str:
        return f"<link {self._rel}{self._href}{self._sizes}{self._type}{self._hreflang}>".replace(
            " >", ">"
        )



from .utilities import ArgumentParser
from .utilities import PyProjectConfig
from .utilities import compile_vite_apps
from .utilities import list_vite_apps
from .utilities import load_vite_apps
from .utilities import subparsers


def cli() -> None:
    from . import __version__

    parser = ArgumentParser(prog="vt", add_help=False)
    parser.add_argument(
        "--version", "-v", action="version", version=f"vite-transporter {__version__}"
    )
    parser.add_argument("--help", "-h", action="help")

    subparsers(parser)

    with PyProjectConfig() as pyproject_config:
        vite_apps_found = load_vite_apps(pyproject_config)
        parsed_args = parser.parse_args()

        if hasattr(parsed_args, "compile"):
            compile_vite_apps(pyproject_config, vite_apps_found, parsed_args)

        if hasattr(parsed_args, "list") or hasattr(parsed_args, "ls"):
            list_vite_apps(vite_apps_found)



import typing as t

from markupsafe import Markup


class BodyContent:
    div_id: str
    noscript_message: str

    def __init__(
        self,
        div_id: str = "root",
        noscript_message: str = "You need to enable JavaScript to run this app.",
    ) -> None:
        self.div_id = div_id
        self.noscript_message = noscript_message

    def __repr__(self) -> str:
        return (
            "BodyContent< "
            f"id = {self.div_id} "
            f"noscript = {self.noscript_message} "
            ">"
        )

    def __str__(self) -> Markup:
        return Markup(self._compile())

    def __call__(self, *args: t.Any, **kwargs: t.Any) -> Markup:
        return Markup(self._compile())

    def _compile(self) -> Markup:
        return Markup(
            f'<div id="{self.div_id}"></div>\n'
            f"<noscript>{self.noscript_message}</noscript>"
        )
